package com.taco.swinger.text2gui.appbase;

import java.beans.*;
import java.io.File;
import java.io.IOException;
import java.util.ResourceBundle;
import java.util.Map;
import java.util.HashMap;
import javax.swing.*;
import java.awt.event.*;

import org.apache.log4j.Logger;

import com.taco.data.INoReturnObservableMap;
import com.taco.util.ChainedResourceBundleFactory;
import com.taco.text.*;
import com.taco.swinger.SwingInvokeProxyFactory;
import com.taco.swinger.text2gui.*;

/** A skeleton class for applications that open and save files. */
public abstract class FileAppBase extends AppBase {
    public FileAppBase() {
        super();           
    }
        
    protected void _addArgMapListeners() {          
        super._addArgMapListeners();
        
        _argMap.addPropertyChangeListener("saveData",
          new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent event) {
                Map map = new HashMap((Map) event.getNewValue());
                map.put("file", _currentFile);                                                
                save(map);
            }
          });        
    }    
    
    public void setModified(boolean flag) {
       _modified = flag;
       _updateRevertState();
    }
        
    public boolean newProject() {
        int option = _shouldSaveFirst();
        if (option == JOptionPane.YES_OPTION) {  
           if (!save(true)) {
               return false;
           }
        } else if (option == JOptionPane.CANCEL_OPTION) { 
           return false; 
        }
        
        _currentFile = null;       
        
        _argMap.putNoReturn("requestNew", null, true);
        
        setModified(false);

        return true;        
    } 
        
    public boolean open() {              
        int option = _shouldSaveFirst();
        if (option == JOptionPane.YES_OPTION) {  
           if (!save(true)) {
               return false;
           }
        } else if (option == JOptionPane.CANCEL_OPTION) { 
           return false; 
        }
            
        // Used previously created version for speed             
        if (_openFileChooser == null) {
            try {
                _openFileChooser = (JFileChooser) 
                DispatchingComponentConverter.DEFAULT_INSTANCE.toObject(
                 _bundle, "openFileChooser", _argMap, null);
            } catch (Exception e) {
                _logger.warn("Can't create open file chooser", e);
                return false;   
            }
            
            // Try to use the current directory that the application is
            // running in.
            _openFileChooser.setCurrentDirectory(new File("."));
        }   
                                   
        if (_lastDir != null) {
            _openFileChooser.setCurrentDirectory(_lastDir);
        }
        
        int x = _openFileChooser.showOpenDialog(_frame);
              
        if (x == JFileChooser.APPROVE_OPTION) {
            _currentFile = _openFileChooser.getSelectedFile();
            _lastDir = _currentFile.getParentFile();            
            return open(_currentFile);            
        }          
        
        return false;
    } 
    
    public abstract boolean open(File file);

    public abstract boolean save(Map options);
    
    /** Discard the current document and revert to the saved version. */
    public boolean revert() {
        if (_currentFile == null) {        
            return false; // FIXME
        }

        INoReturnObservableMap revertArgMap = 
          SwingInvokeProxyFactory.makeSwingInvokeProxyMap();          

        JOptionPane optionPane = null;
        
        try {
            optionPane = (JOptionPane) 
             DispatchingComponentConverter.DEFAULT_INSTANCE.toObject(
             _bundle, "revertOptionPane", revertArgMap, null);
             
             JDialog dialog = optionPane.createDialog(_frame, (String) 
              QuotedStringConverter.instance.toObject(_bundle, "revertDialog.title",
              revertArgMap, null));
             
             dialog.show();
        } catch (Exception e) {
            _logger.warn("Can't create revert option pane", e);
            return false;                
        }
                              
        int choice = ((Integer) optionPane.getValue()).intValue();        
        
        if (choice == JOptionPane.YES_OPTION) {
            return open(_currentFile);            
        }   
        
        return false;
    }

      
    /** Save the current file if available, otherwise call saveAs(). 
     *  If sync is true, be sure to finish saving before returning.
     */
    public boolean save(boolean sync) {
        if (_currentFile == null) {
            return saveAs(sync);                 
        } else {
            return _requestSave(sync);        
        }
    } 
  
    /** Open a Save As dialog to allow the user to choose a file to save to.
     *  After that, save to the chosen file.
     *  If sync is true, be sure to finish saving before returning.
     */  
    public boolean saveAs(boolean sync) {
        // Used previously created version for speed             
        if (_saveFileChooser == null) {
            try {
                _saveFileChooser = (JFileChooser)
                  DispatchingComponentConverter.DEFAULT_INSTANCE.toObject(
                 _bundle, "saveFileChooser", _argMap, null);
            } catch (Exception e) {
                _logger.warn("Can't create save file chooser", e);
                return false;   
            }                 
        }   
                                   
        if (_lastDir != null) {
            _saveFileChooser.setCurrentDirectory(_lastDir);
        }
        
        int x = _saveFileChooser.showSaveDialog(_frame);
              
        if (x == JFileChooser.APPROVE_OPTION) {
            _currentFile = _saveFileChooser.getSelectedFile();
            _lastDir = _currentFile.getParentFile();        
            return _requestSave(sync);
        } else {
            return false;
        }
    }
    
    /** If the state is modified, ask if the user wants to save. The user may
     *  cancel the operation, in which case, this method simply returns.
     *  After deciding to save or not, this method exits the application.
     */
    public void exit() {
        if (_isSaving) {
            System.out.println("Still saving ... ");
            return; // fixme
        } 
        
        int option = _shouldSaveFirst();    
        if (option == JOptionPane.YES_OPTION) { // yes
            // If we failed to save, do not exit.       
            if (!save(true)) {           
                return;
            }
        } else if (option == JOptionPane.CANCEL_OPTION) { // cancel
            return; 
        }
                        
        System.exit(0);    
    }

    public static void start(final AppBase app, final String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                app.initGUI();
                if (args.length > 0) {
                    ((FileAppBase) app).open(new File(args[0]));
                }
            }
        });        
    }            
    
    protected void _updateRevertState() {        
        boolean canRevert = (_currentFile != null) && _modified;
       
        if (_canRevert != canRevert) {
            _canRevert = canRevert;            
            _argMap.putNoReturn("canRevert", new Boolean(canRevert), true);
        }
    }
          
    protected int _shouldSaveFirst() {    
        if (_modified) {           
            try {
                JOptionPane saveBeforePane = (JOptionPane) 
                  DispatchingComponentConverter.DEFAULT_INSTANCE.toObject(
                 _bundle, "saveBeforePane", _argMap, null);        
                JDialog dialog = saveBeforePane.createDialog(_frame,  
                 (String) QuotedStringConverter.instance.toObject(
                 _bundle, "modifiedFileDialog.title", _argMap, null));            
                dialog.show();                                     
                
                return ((Integer) saveBeforePane.getValue()).intValue();
            } catch (Exception e) {
                _logger.warn("Can't create save before option pane", e);
                return JOptionPane.CANCEL_OPTION;                
            }
        }      
        return JOptionPane.NO_OPTION;        
    }    
    
    protected boolean _requestSave(boolean sync) {
        try {
            _argMap.putNoReturn("gatherSaveData", new Boolean(sync), true);
            return true;
        } catch (Exception e) {
            return false;   
        }
    }
   
    protected boolean _modified = false;
    protected boolean _canRevert = false;    
    protected JFileChooser _openFileChooser = null;
    protected JFileChooser _saveFileChooser = null;
    protected File _currentFile = null;
    protected File _lastDir = null;
    protected boolean _isSaving = false;           

    private static final Logger _logger =
      Logger.getLogger(FileAppBase.class);
}
